package com.chelvc.framework.base.interceptor;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ReadListener;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.common.util.ObjectUtils;

/**
 * 请求参数缓存拦截器
 *
 * @author Woody
 * @date 2025/6/25
 */
public class RequestCachingInterceptor implements Filter {
    /**
     * 缓存数据流对象
     */
    static class Stream extends ServletInputStream {
        private final InputStream delegate;

        private Stream(byte[] bytes) {
            this.delegate = new ByteArrayInputStream(bytes);
        }

        @Override
        public int read() throws IOException {
            return this.delegate.read();
        }

        @Override
        public int read(byte[] b, int off, int len) throws IOException {
            return this.delegate.read(b, off, len);
        }

        @Override
        public int available() throws IOException {
            return this.delegate.available();
        }

        @Override
        public boolean isReady() {
            try {
                return this.delegate.available() > 0;
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public boolean isFinished() {
            try {
                return this.delegate.available() == 0;
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public void setReadListener(ReadListener listener) {
            throw new UnsupportedOperationException();
        }
    }

    /**
     * 请求缓存包装对象
     */
    static class Wrapper extends HttpServletRequestWrapper {
        private final byte[] body;
        private final String query;
        private final Map<String, String[]> parameters;

        private Wrapper(HttpServletRequest request) throws IOException {
            super(request);
            if (request instanceof Wrapper) {
                Wrapper wrapper = (Wrapper) request;
                this.body = wrapper.body;
                this.query = wrapper.query;
                this.parameters = wrapper.parameters;
            } else {
                try (InputStream input = request.getInputStream()) {
                    this.body = FileUtils.getBytes(input);
                }
                this.query = request.getQueryString();
                this.parameters = request.getParameterMap();
            }
        }

        @Override
        public String getQueryString() {
            return this.query;
        }

        @Override
        public String getParameter(String name) {
            String[] values = this.parameters.get(name);
            return ObjectUtils.isEmpty(values) ? null : values[0];
        }

        @Override
        public Map<String, String[]> getParameterMap() {
            return this.parameters;
        }

        @Override
        public Enumeration<String> getParameterNames() {
            return new Enumeration<String>() {
                private final Iterator<String> iterator = parameters.keySet().iterator();

                @Override
                public boolean hasMoreElements() {
                    return this.iterator.hasNext();
                }

                @Override
                public String nextElement() {
                    return this.iterator.next();
                }
            };
        }

        @Override
        public String[] getParameterValues(String name) {
            return this.parameters.get(name);
        }

        @Override
        public BufferedReader getReader() throws IOException {
            return new BufferedReader(new InputStreamReader(this.getInputStream()));
        }

        @Override
        public ServletInputStream getInputStream() throws IOException {
            return new Stream(this.body);
        }
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        if (request instanceof HttpServletRequest) {
            chain.doFilter(new Wrapper((HttpServletRequest) request), response);
        } else {
            chain.doFilter(request, response);
        }
    }
}
